--***************************************************************
--* File         : TB_WBRAM.vhd                                 *
--* Created      : 2009/11/23 17:38:10 kawasan                  *
--* Modified     : Time-stamp: <09/11/23 21:53:00 kawasan>      *
--*								*
--* Test-Bench                                                  *
--*								*
--* Copyright 2009 Osamu Kawashima. all rights reserved		*
--***************************************************************

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

use std.textio.all;
use ieee.std_logic_textio.all;

entity TB_WBRAM is end;

architecture behavior of TB_WBRAM is
    
	component WBRAM
	generic (
		ADDRESS_WIDTH	: integer := 10;	-- Spartan2(4096bit/BRAM=8) Spartan3(18432bit/BRAM=10)
		DATA_WIDTH      : integer := 16
	);
	port (
		-- WISHBONE SLAVE interface:
		WB_CLK_I        : in    std_logic;
		WB_RST_I        : in    std_logic;
		WB_ADR_I        : in    std_logic_vector(31 downto 0);
		WB_DAT_I        : in    std_logic_vector(31 downto 0);
		WB_WE_I         : in    std_logic;
		WB_STB_I        : in    std_logic;
		WB_ACK_O        : out   std_logic;
		WB_DAT_O        : out   std_logic_vector(31 downto 0)
	);
	end component;

	constant ADDRESS_WIDTH	: integer := 10;	-- Spartan2(4096bit/BRAM=8) Spartan3(18432bit/BRAM=10)
	constant DATA_WIDTH     : integer := 16;

	constant ZERO32         : std_logic_vector(31 downto 0) := (others=>'0');
	constant ONE32          : std_logic_vector(31 downto 0) := (others=>'1');

	constant WT1            : Time := 10 ns;
	constant WT_DELAY       : Time := 1 ns;
	constant WT_IDLE        : Time := 100 ns;
	constant CLK_CYCLE      : Time := 21 ns;	-- about 48MHz

	signal WB_CLK_I         : std_logic := '0';
	signal WB_RST_I         : std_logic := '1';
	signal WB_ADR_I         : std_logic_vector(31 downto 0) := (others=>'0');
	signal WB_DAT_I         : std_logic_vector(31 downto 0) := (others=>'0');
	signal WB_WE_I          : std_logic := '0';
	signal WB_STB_I         : std_logic := '0';
	signal WB_ACK_O         : std_logic := '0';
	signal WB_DAT_O         : std_logic_vector(31 downto 0) := (others=>'0');

	signal rd_data          : std_logic_vector(WB_DAT_O'range) := (others=>'0');
	signal rd_dat           : std_logic_vector(WB_DAT_O'range) := (others=>'0');
	signal wr_dat           : std_logic_vector(WB_DAT_I'range) := (others=>'0');

	-- WB Read (32bit)
	procedure rd_uw(
		ADD_I		: in    std_logic_vector(31 downto 0);
		DAT_I		: in    std_logic_vector(31 downto 0);
	signal	ADD_O		: out   std_logic_vector(31 downto 0);
	signal	DAT_O		: out   std_logic_vector(31 downto 0);
	signal	STB		: out   std_logic;
	signal	WE		: out   std_logic
	) is
	begin
		wait until WB_CLK_I'event and WB_CLK_I='1';
		wait for WT_DELAY;
		ADD_O	<= ADD_I;
		STB	<= '1';
		WE	<= '0';
		wait until WB_CLK_I'event and WB_CLK_I='1';
		wait until WB_CLK_I'event and WB_CLK_I='1';
		wait for WT_DELAY;
		DAT_O	<= DAT_I;
		wait for WT_DELAY;
		ADD_O	<= (others=>'0');
		STB	<= '0';
		WE	<= '0';
	end rd_uw;

	-- WB Write (32bit)
	procedure wr_uw(
		ADD_I		: in    std_logic_vector(31 downto 0);
		DAT_I		: in    std_logic_vector(31 downto 0);
	signal	ADD_O		: out   std_logic_vector(31 downto 0);
	signal	DAT_O		: out   std_logic_vector(31 downto 0);
	signal	STB		: out   std_logic;
	signal	WE		: out   std_logic
	) is
	begin
		wait until WB_CLK_I'event and WB_CLK_I='1';
		wait for WT_DELAY;
		ADD_O	<= ADD_I;
		DAT_O	<= DAT_I;
		STB	<= '1';
		WE	<= '1';
		wait until WB_CLK_I'event and WB_CLK_I='1';
		wait for WT_DELAY;
		ADD_O	<= (others=>'0');
		DAT_O	<= (others=>'0');
		STB	<= '0';
		WE	<= '0';
	end wr_uw;

begin
    
	uWBRAM : WBRAM
	generic map (
		ADDRESS_WIDTH	=> ADDRESS_WIDTH,
		DATA_WIDTH      => DATA_WIDTH
	)
	port map (
		WB_CLK_I        => WB_CLK_I,
		WB_RST_I        => WB_RST_I,
		WB_ADR_I        => WB_ADR_I,
		WB_DAT_I        => WB_DAT_I,
		WB_WE_I         => WB_WE_I,
		WB_STB_I        => WB_STB_I,
		WB_ACK_O        => WB_ACK_O,
		WB_DAT_O        => WB_DAT_O
	);

	process(WB_RST_I, WB_CLK_I)
	begin
		if WB_RST_I='1' then
			rd_dat <= (others=>'0');
		elsif WB_CLK_I'event and WB_CLK_I='1' then
			if WB_STB_I='1' and WB_WE_I='0' and WB_ACK_O='1' then
				rd_dat <= WB_DAT_O;
			else
				rd_dat <= rd_dat;
			end if;
		end if;
	end process;

	process begin
		wait for CLK_CYCLE/2;
		WB_CLK_I <= not WB_CLK_I;
	end process;

	process
		variable ln         : line;
	begin
		-- Reset
		wait for WT_IDLE+WT_DELAY;
		WB_RST_I <= '0';
		wait for WT_IDLE;

		-- End
		write(ln, string'("--- End of Test-Bench. ---"));
		writeline(output, ln);
		wait;
	end process;

	process begin
		-- Reset
		wait for WT_IDLE+WT_DELAY;

		-- Write 3
		wait for WT_IDLE*2;
		wr_uw(x"00000000", x"000155AA", WB_ADR_I, WB_DAT_I, WB_STB_I, WB_WE_I);
		wr_uw(x"00000010", x"0002AA55", WB_ADR_I, WB_DAT_I, WB_STB_I, WB_WE_I);
		wr_uw(x"00000080", x"00031234", WB_ADR_I, WB_DAT_I, WB_STB_I, WB_WE_I);

		-- No.2
		wait for WT_IDLE*2;
		rd_uw(x"00000000", WB_DAT_O, WB_ADR_I, rd_data, WB_STB_I, WB_WE_I);
		rd_uw(x"00000010", WB_DAT_O, WB_ADR_I, rd_data, WB_STB_I, WB_WE_I);
		rd_uw(x"00000080", WB_DAT_O, WB_ADR_I, rd_data, WB_STB_I, WB_WE_I);

		-- No.3
		wait for WT_IDLE*2;
		wr_uw(x"00000058", x"01020304", WB_ADR_I, WB_DAT_I, WB_STB_I, WB_WE_I);
		rd_uw(x"00000058", WB_DAT_O, WB_ADR_I, rd_data, WB_STB_I, WB_WE_I);

		-- End
		wait;
	end process;

end behavior;
